// module hu_q.h
// defines the classes HU_Q and HU_QE implementing a FIFO queue
// Copyright Dr.E.Huckert 2003

// Versions:
// 01.06.2010 Queues sind jetzt binaerfaehig

#ifndef HUQ_DEFINED
#define HUQ_DEFINED

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define HUQ_MAXENTRIES 50

// ----------------------------------------------------------------
// an element of a linked list
// contains a pointer to the successor and the stored content
class HU_QE
{
private:
  HU_QE *pNext;    // pointer to successor element
  char *pContent;  // must be allocated with new - no reference allowed!
  int  lContent;   // Laenge des Inhalts
public:
  HU_QE(const char *content);
  HU_QE(const char *content, int lContent);
  ~HU_QE();
  HU_QE *getNext()
  { return this->pNext; };
  char *getContent()
  { return this->pContent; };
  int getContentLength()
  { return this->lContent; };
  void setNext(HU_QE *pn)
  { this->pNext = pn; };
};

// ----------------------------------------------------------------
// a doubly linked list 
class HU_Q
{
private:
  HU_QE      *pStart;      // Pointer to first element
  unsigned   maxEntries;
  unsigned   noEntries;
  void       removeLast(HU_QE *pe);
public:
  HU_Q();
  HU_Q(unsigned max_entries);
  ~HU_Q();
  void clear();
  HU_QE *getStart();
  HU_QE *getEnd();
  HU_QE *getPrevious(HU_QE *pe);
  char *get();
  char *get(int *pLength);
  int add(const char *ps);
  int add(const char *ps, int lps);
  unsigned getNoEntries();
};   // end HU_Q
#endif
