// module hu_trace.cpp
// Dr.E.Huckert 02-2005

// Note : pointer to HU_TRACE_ADMIN must be declared in the main application:
//    HU_TRACE_ADMIN *pTraceAdmin = NULL;

// Versions:
//  07-2006: two alternative implementations: file based with immediate
//           write-thru to file or
//           list based (with serialisation in the destructor)
//  02-2008: include Dateien ueberarbeitet
//  03-2008  logging via UDP added, list based logging dropped
//           A timestamp is now always added in front of the message
//  11-2008  trace administration (class HU_TRACE_ADMIN) added
//           Trace output may be enabled/disabled

#include "hu_trace.h"

#ifdef WIN32
// unbedingt fr BCB winsock2.h vor(!) winsock.h includen!
WSADATA WSAdata;
#else
using namespace std;
#endif

// --------------------------------------------------------
// Constructor
HU_TRACE_ADMIN::HU_TRACE_ADMIN()
{
  int n;
  for (n=0; n < 32; n++)
  {
    this->pointers[n] = NULL;
    this->ids[n]      = NULL;
  }
  this->noRegistered = 0;
}

// --------------------------------------------------------
// Register an instance of class HU_TRACE
void HU_TRACE_ADMIN::Register(HU_TRACE *pTrace)
{
  if (pTrace == NULL) return;
  if (this->noRegistered < 16)
  {
    this->pointers[this->noRegistered] = pTrace;
    this->ids[this->noRegistered]      = pTrace->getId();
    this->noRegistered++;
  }
}  

// --------------------------------------------------------
// Allow/disallow the trace capabilities
void HU_TRACE_ADMIN::allowAll(int flag)
{
  int n;
  HU_TRACE *pt;
  for (n=0; n < this->noRegistered; n++)
  {
    pt = this->pointers[n];
    if (pt != NULL) pt->setEnabled(flag);
  }
}

// --------------------------------------------------------
// Allow/disallow the trace capabilities for an instance with 
// the given instance pointer
void HU_TRACE_ADMIN::allow(HU_TRACE *pt, int flag)
{
  if (pt == NULL) return;
  pt->setEnabled(flag);
}

// ---------------------------------------------------------
int HU_TRACE_ADMIN::isAllowed(HU_TRACE *pt)
{
  if (pt == NULL) return 0;
  return pt->getEnabled();
}   

// --------------------------------------------------------
// ------------------------------------------------------------------------
// Alle Konstruktoren muessen diese Methode aufrufen!
void HU_TRACE::initialize(const char *id,
                          const char *fn,
                          const char *udpserver,
                          const int udpport)
{
  this->error = 0;
  this->sock = -1;
  this->enabled = 1;
#ifdef WIN32
  if (::crit_trace_initialized == 0)
  {
    ::crit_trace_initialized = 1;
    InitializeCriticalSection(&::crit_trace);
  }
#endif
  this->setLock();
/*
  if (::pTraceAdmin == NULL)
    ::pTraceAdmin = new HU_TRACE_ADMIN();
  // register this instance in the global administration
  if (::pTraceAdmin != NULL)
        ::pTraceAdmin->Register(this);
*/
  if (id != NULL)
  {
    this->idStr = new char[::strlen(id) + 1];
    ::strcpy(idStr, id);
  }
  else
  {
    this->idStr = new char[1];
    idStr[0] = 0;
  }    
  if (fn != NULL)
  {
    // file based
    this->ofd = creat(fn, 0777);
    this->setFileName(fn);
    this->count = 0L;
    this->udpServer = NULL;
    this->udpPort   = -1;
    this->sock = -1;
  }
  else
  {
    // UDP based
    #ifdef WIN32
    if (WSAStartup(0x0101, &(WSAdata)) != 0)
    {
      this->error++;
      this->unLock();
      return;
    }
  #endif
    udpServer = new char[128];
    strcpy(this->udpServer, udpserver);
    this->udpPort = udpport;
    this->filename = NULL;
    this->ofd = -1;
    this->sock = this->create_socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
    {
      this->error++;
    }
  }
  this->unLock();  
} // end HU_TRACE::initialize()

// ------------------------------------------------------------------------
HU_TRACE::HU_TRACE()
{
  this->initialize("", "hu_trace.log", NULL, -1);
}

// ------------------------------------------------------------------------
// standard constructor for file based logging
HU_TRACE::HU_TRACE(const char *fn)
{
   this->initialize("", fn, NULL, -1);
}

// ------------------------------------------------------------------------
// constructor for UDP based logging
// A trace administration is built up only when this form of logging
// is used!
HU_TRACE::HU_TRACE(const char *id, const char * udpserver, int udpport)
{
  this->initialize(id, NULL, udpserver, udpport);
}

// ------------------------------------------------------------------------
// constructor for UDP based logging
// Evaluates env. var. UDPSERVER, assumes "localhost" as default if not set
HU_TRACE::HU_TRACE(const char *id,  int udpport)
{
  static char s_localhost[16] = "localhost";
  char *udpserver;
  udpserver = ::getenv("UDPSERVER");
  if (udpserver == NULL) udpserver = s_localhost;
  else
  {
    if (::strlen(udpserver) == 0) udpserver = s_localhost;
  }
  this->initialize(id, NULL, udpserver, udpport);
}

// --------------------------------------------------------
// Returns an instance to the trace administration
// Note: HU_TRACE_ADMIN is a singleton!
// May return NULL!
HU_TRACE_ADMIN *HU_TRACE::getAdmin()
{
  //return pTraceAdmin;
  return NULL;
}

// --------------------------------------------------------
char *HU_TRACE::getId()
{
  return idStr;
}

// --------------------------------------------------------
// liefert Zeit im Format SS:MM:SS:mmm 
// The returned pointer must be freed externally!
char *HU_TRACE::getTime()
{
  time_t   ltime;
  struct tm *newtime;
  struct timeb tb;
  char *retbuf;
  time(&ltime);
  newtime = localtime(&ltime);
  ::ftime(&tb);
  retbuf = new char[16];
  if (retbuf != NULL)
  {
    sprintf(retbuf,"%02d:%02d:%02d.%03d",
          (int)newtime->tm_hour,
          (int)newtime->tm_min,
          (int)newtime->tm_sec,
          (int)tb.millitm);
  }
  return retbuf;
}   // end get_time()

// ------------------------------------------------------  
// convert a binary IP address (IPV4) to a string
// Warning: this is not reentrant!
char *HU_TRACE::convert_binary_ip(char *binstr)
{
int n, ic;
char xbuf[8];
static char retbuf[32];
  retbuf[0] = 0;
  for (n=0; n < 4; n++)
  {
    ic = binstr[n] & 0xff;
    ::sprintf(xbuf, "%0d", ic);
    ::strcat(retbuf, xbuf);
    if (n < 3) ::strcat(retbuf, ".");
  }
  return retbuf;
}   // end convert_binary_ip()

// -------------------------------------------------------
int HU_TRACE::UDP_send(const char *data,
                       const size_t size)
{
  struct sockaddr_in addr_sendto;
  struct hostent *h;
  int    rc;
  char   *caddr;
  char   inetaddr[32];
  if (this->sock == -1)
    return -1;
  //
  // resolve the servername into an IP address
  h = ::gethostbyname(this->udpServer);
  if (h == NULL)
    return -2;
  char **pptr;
  for (pptr = h->h_addr_list; *pptr != NULL; pptr++)
  {
    caddr = this->convert_binary_ip(*pptr); 
    ::strcpy(inetaddr, caddr);
  }
  ::memset((char *)&addr_sendto, 0, sizeof(addr_sendto));
  addr_sendto.sin_family      = AF_INET;
  addr_sendto.sin_port        = htons(this->udpPort);
  addr_sendto.sin_addr.s_addr = inet_addr(inetaddr);
  //
  // send now
  rc = ::sendto(this->sock, data, size, 0,
                (struct sockaddr *)&addr_sendto,
                sizeof (addr_sendto));
  return rc;
}   // end HU_TRACE::UDP_send()

// -------------------------------------------------------
sock_t HU_TRACE::create_socket(int af, int type, int protocol)
{
    sock_t sock;
    const int y = 1;
    sock = ::socket(af, type, protocol);
    if (sock < 0) return sock;
    ::setsockopt( sock, SOL_SOCKET,
                SO_REUSEADDR, (char *)(&y), sizeof(int));
    return sock;
}   // end HU_TRACE::create_socket()

// -------------------------------------------------------
void HU_TRACE::close_socket()
{
  if (this->sock != -1)
#ifdef WIN32
    ::closesocket(this->sock);
#else
    ::close(this->sock);
#endif
}

// -------------------------------------------------------
void HU_TRACE::cleanup(void)
{
#ifdef WIN32
   /* Cleanup Winsock */
   WSACleanup();
#endif
}   // end HU_TRACE::cleanup()

// ------------------------------------------------------------------------
void HU_TRACE::setLock()
{
#ifdef WIN32
  EnterCriticalSection(&::crit_trace);
#else
#endif
}

// ------------------------------------------------------------------------
void HU_TRACE::unLock()
{
#ifdef WIN32
  LeaveCriticalSection(&::crit_trace);
#else
#endif
}

// ------------------------------------------------------------------------
// Destructor
HU_TRACE::~HU_TRACE()
{
  if (this->idStr != NULL)
    delete this->idStr;
  this->idStr = NULL;
  if (this->filename != NULL)
  {
    close(ofd);
    delete this->filename;
  }
  else
  {
    if (udpServer != NULL)
    {
      this->close_socket();
      this->cleanup();
      delete udpServer;
    }
    udpServer = NULL;
  }
  this->count = 0L;
}

// ------------------------------------------------------------------------
void HU_TRACE::setFileName(const char *fn)
{
  this->filename = new char[strlen(fn) + 1];
  strcpy(this->filename, fn);
}

// -------------------------------------------------------------------------
// Returns < 0 upon error
int HU_TRACE::UDP_send(const char *msg)
{
  return 0;   // ok
}

// ------------------------------------------------------------------------
// This is the central logging routine
// Assumes that the messages have normally no LF
int HU_TRACE::trace(const char *msg)
{
  if (! this->enabled)
    return -1;
  this->setLock();
  int ret = -1;
  int lMsg = ::strlen(msg);
  int lId  = ::strlen(this->idStr);
  char *pTime = this->getTime();  // returns format HH:MM:SS.mmm
  char *pMsg = new char[lMsg + lId + 12 + 3 + 1];
  if ((pMsg != NULL) && (pTime != NULL))
  {
    sprintf(pMsg, "%s %s %s\n", pTime, this->idStr, msg);
    lMsg = lMsg + lId + 12 + 3;
    //
    if (this->ofd >= 0)
    {
      // logging is file based
      if (msg != NULL)
      {
        this->count++;
        ret = ::write(this->ofd, pMsg, lMsg);
      }
    }
    else
    {
      // logging is UDP based
      ret = this->UDP_send(pMsg, lMsg); 
    }
    delete pMsg;
    delete pTime;
  }
  this->unLock();
  return ret;
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
// appends a LF at the end of the string to be written
void HU_TRACE::traceLF(const char *msg)
{
  if (msg != NULL)
  {
    char *aux;
    aux = new char[strlen(msg) + 4];
    strcpy(aux, msg);
    strcat(aux, "\n");
    this->trace(aux);
    delete aux;
  }
}

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, const char *arg1)
{
  if ((msg != NULL) && (arg1 != NULL))
  {
    //wxString s1(msg);
    //s1.append(arg1);
    //this->trace(s1.c_str());
    int l1 = strlen(msg);
    int l2 = strlen(arg1);
    char *aux = new char[l1 + l2 + 10];
    strcpy(aux, msg);
    strcat(aux, arg1);
    this->trace(aux);
    delete aux;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, int arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%d", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, int arg1, int arg2)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%d %d", arg1, arg2);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, long arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%ld", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, float arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    sprintf(aux, "%f", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, unsigned int arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%u", arg1);
    caddr = new char[::strlen(msg) + ::strlen(aux) + 20];
    ::strcpy(caddr, msg);
    ::strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, unsigned long arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%lu", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::tracex(const char *msg, long arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%08lx", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *msg, const char arg1)
{
  if (msg != NULL)
  {
    char *caddr;
    char aux[32];
    SPRINTF(aux, "%c", arg1);
    caddr = new char[strlen(msg) + strlen(aux) + 20];
    strcpy(caddr, msg);
    strcat(caddr, aux);
    this->trace(caddr);
    delete caddr;
  }
}   // end HU_TRACE::trace()

/*
// ------------------------------------------------------------------------
void HU_TRACE::trace(const STRING msg)
{
  this->trace(msg.c_str());
}

// ------------------------------------------------------------------------
void HU_TRACE::trace(const STRING *pmsg)
{
  if (pmsg != NULL)
  {
    this->trace(pmsg->c_str());
  }
}

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *pstr, const STRING msg)
{
  if (pstr != NULL)
  {
    this->trace(pstr, msg.c_str());
  }
}

// ------------------------------------------------------------------------
void HU_TRACE::trace(const char *pstr, const STRING *pmsg)
{
  if (pstr != NULL)
  {
    this->trace(pstr, pmsg->c_str());
  }
}
*/
