// module hu_trace.h
// Dr.E.Huckert 2005

// Note : pointer to HU_TRACE_ADMIN must be declared in the main application:
//    HU_TRACE_ADMIN *pTraceAdmin = NULL;

// Versions:
// 13.03.2008 logging via UDP added, list based logging dropped
// 16.11.2008 redesigned for Linux

#ifndef HUTRACE_DEFINED
#define HUTRACE_DEFINED

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#ifdef WIN32
#include <winsock2.h>     // muss f�r BCB unbedingt vor(!) winsock.h stehen!
#include <winsock.h>
#include <windows.h>
#include <io.h>
#define SPRINTF wsprintf
#else
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <resolv.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/msg.h>
#include <netinet/in.h>
#define SPRINTF sprintf
#endif

// for Borland CBuilder6
//#define STRING AnsiString

#ifdef USE_STL
#include <string>
#define STRING string
#endif

#ifdef WXW
#include <wx/wx.h>
#define STRING wxString
#include <wx/textfile.h>
#endif

#define sock_t int

//#include "mutex.h"

#ifdef WIN32
static  CRITICAL_SECTION crit_trace;
static  int crit_trace_initialized = 0;
#else
using namespace std;
#endif

class HU_TRACE_ADMIN;   // forward decl.

class HU_TRACE
{
private:
  char *filename;            // for file based logging
  int  ofd;                  // for file based logging
  char *udpServer;           // for UDP based logging
  int  udpPort;              // for UDP based logging
  sock_t sock;
  unsigned long count;
  int error; 
  char *idStr;
  int enabled;
  //
  void initialize(const char *id, const char *fn, const char *udpserver, int port);
  int UDP_send(const char *msg);
  void cleanup(void);
  sock_t create_socket(int af, int type, int protocol);
  void close_socket();
  char * convert_binary_ip(char *binstr);
  int UDP_send(const char *data,  const size_t size);
  char *getTime();

public:
  HU_TRACE();
  HU_TRACE(const char *fn);
  HU_TRACE(const char *id, const char *udpserver, int udpport);
  HU_TRACE(const char *id, int udpport);
  ~HU_TRACE();
  void setLock();
  void unLock();
  int  trace(const char *msg);
  void traceLF(const char *msg);
  void trace(const char *msg, const char *arg1);
  void trace(const char *msg, int arg1);
  void trace(const char *msg, int arg1, int arg2);
  void trace(const char *msg, unsigned int arg1);
  void trace(const char *msg, long arg1);
  void trace(const char *msg, float arg1);
  //void trace(DL_LIST *plist, const char *msg1, const char *msg2);
  void tracex(const char *msg, long arg1);
  void trace(const char *msg, unsigned long arg1);
  void setFileName(const char *fn);
  char *getId();
  /*
  void trace(const STRING str);
  void trace(const STRING *pstr);
  void trace(const char *str1, const STRING str2);
  void trace(const char *str1, const STRING *pstr2);
  */
  void trace(const char *str1, const char ch);
  void setEnabled(int flag)
  { this->enabled = flag; };
  int  getEnabled()
  { return this->enabled; };
  HU_TRACE_ADMIN *getAdmin();
};

// global HU_TRACE administration
// HU_TRACE_ADMIN is a singleton!
// Allows central switch on/switch off etc.
class HU_TRACE_ADMIN
{
  private:
    HU_TRACE *pointers[32];
    char     *ids[32];
    //
    int noRegistered;
  public:
    HU_TRACE_ADMIN();
    void Register(HU_TRACE *p);
    void allowAll(int flag);
    void allow(HU_TRACE *pt, int flag);
    int isAllowed(HU_TRACE *pt);
    int getNoRegistered()
    { return this->noRegistered; };
    HU_TRACE *getInstance(int no)
    {  return this->pointers[no]; };
};

extern HU_TRACE_ADMIN *pTraceAdmin;
#endif
