// module mutex.h
// Copyright Dr.E.Huckert 6-2000
// Note: the class FILE_MUTEX is only implemented for UNIX (not WINDOWS)

// Versions
// 20.03.2009 FILE_Mutex::setlock() and unlock() return now an int value
//            method now returns immediately
//            new method FILE_Mutex::setlockWait(): waits!

#ifndef MUTEX_DEFINED
#define MUTEX_DEFINED

//#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef WIN32
#include <poll.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/shm.h>
#include <errno.h>
#include <signal.h>
#include <semaphore.h>
#include <sys/wait.h>
#include <sys/sem.h>
#include <sys/types.h>
#else
#include <windows.h>
#include <io.h>
#include <winsock.h>
#include <process.h>
#endif

extern int errno;

#ifdef WIN32
#define Mutex_TYPE HANDLE
#else
#define Mutex_TYPE pthread_mutex_t
#endif

// -----------------------------------------------------
// this class defines an exception object
class EXC_EH
{
public:
  EXC_EH(int exc_code)
  {
    //cerr << "Exception " << exc_code << " raised" << endl;
  }
};
// ---------------------------- class Mutex ----------------------------
class Mutex
{
private:
  int  owner;
  int  logging;
  char mut_name[17];
  Mutex_TYPE mutex;
public:
  Mutex(const char *name);
  ~Mutex();
  long setlock(int fd);   // for file mutexes
  long unlock(int fd);
  long setlock();
  long unlock();
  void setLogging(int flag);
  char *get_name()
  {  return this->mut_name; };
  Mutex_TYPE get_mutex()
  {  return this->mutex; };
  long get_thread();
};

// ---------------------------------------
// a file mutex based on fcntl
class FILE_Mutex
{
private:
  int fd;   // the file descriptor
  int logging;
public:
  FILE_Mutex(int fileid);
  ~FILE_Mutex();
  int setlock();
  int setlockWait();
  int unlock();
  void setLogging(int flag);
};
#endif
